# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime

class RollInterestUpdates(BaseModel):
    """
    Describes changes to the interest of a FlexibleDeposit instrument as the result of a DepositRollEvent.  Both the interest to be withdrawn and the interest to be reinvested must be specified (either as an amount or as a percentage).  # noqa: E501
    """
    withdraw_interest_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The amount of interest that should be withdrawn from a FlexibleDeposit as the result of a roll event.", alias="withdrawInterestAmount")
    withdraw_interest_percentage: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The percentage of interest that should be withdrawn from a FlexibleDeposit instrument as the result of a roll event.", alias="withdrawInterestPercentage")
    reinvest_interest_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The amount of interest that should be reinvested in a FlexibleDeposit instrument as the result of a roll event.", alias="reinvestInterestAmount")
    reinvest_interest_percentage: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The percentage of interest that should be reinvested in a FlexibleDeposit instrument as the result of a roll event.", alias="reinvestInterestPercentage")
    __properties = ["withdrawInterestAmount", "withdrawInterestPercentage", "reinvestInterestAmount", "reinvestInterestPercentage"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> RollInterestUpdates:
        """Create an instance of RollInterestUpdates from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if withdraw_interest_amount (nullable) is None
        # and __fields_set__ contains the field
        if self.withdraw_interest_amount is None and "withdraw_interest_amount" in self.__fields_set__:
            _dict['withdrawInterestAmount'] = None

        # set to None if withdraw_interest_percentage (nullable) is None
        # and __fields_set__ contains the field
        if self.withdraw_interest_percentage is None and "withdraw_interest_percentage" in self.__fields_set__:
            _dict['withdrawInterestPercentage'] = None

        # set to None if reinvest_interest_amount (nullable) is None
        # and __fields_set__ contains the field
        if self.reinvest_interest_amount is None and "reinvest_interest_amount" in self.__fields_set__:
            _dict['reinvestInterestAmount'] = None

        # set to None if reinvest_interest_percentage (nullable) is None
        # and __fields_set__ contains the field
        if self.reinvest_interest_percentage is None and "reinvest_interest_percentage" in self.__fields_set__:
            _dict['reinvestInterestPercentage'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> RollInterestUpdates:
        """Create an instance of RollInterestUpdates from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return RollInterestUpdates.parse_obj(obj)

        _obj = RollInterestUpdates.parse_obj({
            "withdraw_interest_amount": obj.get("withdrawInterestAmount"),
            "withdraw_interest_percentage": obj.get("withdrawInterestPercentage"),
            "reinvest_interest_amount": obj.get("reinvestInterestAmount"),
            "reinvest_interest_percentage": obj.get("reinvestInterestPercentage")
        })
        return _obj

RollInterestUpdates.update_forward_refs()
