# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg

from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime


class VendorLibrary(str, Enum):
    """
    VendorLibrary
    """

    """
    allowed enum values
    """
    LUSID = 'Lusid'
    REFINITIVQPS = 'RefinitivQps'
    REFINITIVTRACSWEB = 'RefinitivTracsWeb'
    VOLMASTER = 'VolMaster'
    ISDACDS = 'IsdaCds'
    YIELDBOOK = 'YieldBook'
    LUSIDCALC = 'LusidCalc'

    @classmethod
    def from_json(cls, json_str: str) -> VendorLibrary:
        """Create an instance of VendorLibrary from a JSON string"""
        return VendorLibrary(json.loads(json_str))
