# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg

from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime


class ResultValueType(str, Enum):
    """
    Enum of possible result value type. Used discriminate the result values
    """

    """
    allowed enum values
    """
    RESULTVALUE = 'ResultValue'
    RESULTVALUEDICTIONARY = 'ResultValueDictionary'
    RESULTVALUE0D = 'ResultValue0D'
    RESULTVALUEDECIMAL = 'ResultValueDecimal'
    RESULTVALUEINT = 'ResultValueInt'
    RESULTVALUESTRING = 'ResultValueString'
    RESULTVALUEBOOL = 'ResultValueBool'
    RESULTVALUECURRENCY = 'ResultValueCurrency'
    CASHFLOWVALUE = 'CashFlowValue'
    CASHFLOWVALUESET = 'CashFlowValueSet'
    RESULTVALUELIFECYCLEEVENTVALUE = 'ResultValueLifeCycleEventValue'
    RESULTVALUEDATETIMEOFFSET = 'ResultValueDateTimeOffset'

    @classmethod
    def from_json(cls, json_str: str) -> ResultValueType:
        """Create an instance of ResultValueType from a JSON string"""
        return ResultValueType(json.loads(json_str))
